package de.twenty11.skysail.common.navigation;

import java.util.Set;

import org.codehaus.jackson.annotate.JsonIgnore;
import org.restlet.Request;
import org.restlet.data.Form;
import org.restlet.data.Reference;

/**
 * Providing means of navigation is a secondary responsibility of skysail, i.e. skysail will try to gather related
 * information but will not complain if it is not available or determinable.
 * 
 */
public class Navigation {

    private String home;

    @JsonIgnore
    private Request restletRequest;

    private String back;

    private String up;

    private Set<String> mappings;

    public Navigation() {
    }

    public Navigation(Request request, Set<String> mappings) {
        this.restletRequest = request;
        this.mappings = mappings;
        if (request != null) {
            String defaultQueryToAttach = getDefaultQuery(request);
            // if (request.getResourceRef() != null) {
            // resourceRef = request.getResourceRef().toString();
            // resourceRef += defaultQueryToAttach;
            // }
            // hostRef = request.getHostRef() != null ? request.getHostRef().toString() : null;
            if (request.getRootRef() != null) {
                home = request.getRootRef().toString();
                home += defaultQueryToAttach;
            }
            // rootRef = request.getRootRef() != null ? request.getRootRef().toString() : null;
            // originalRef = request.getOriginalRef() != null ? request.getOriginalRef().toString() : null;
            up = calcUpReference(request.getOriginalRef(), request.getRootRef(), mappings, defaultQueryToAttach);
            back = request.getReferrerRef() != null ? request.getReferrerRef().toString() : null;
        }
    }

    private String calcUpReference(Reference reference, Reference rootRef, Set<String> mappings,
            String defaultQueryToAttach) {
        String result = null;
        if (rootRef == null) {
            return null;
        }
        if (reference != null && reference.getParentRef() != null) {
            String upCandidate = reference.getParentRef().toString();
            if (inMappings(mappings, rootRef, upCandidate)) {
                result = upCandidate + defaultQueryToAttach;
            } else {
                if (reference.toString().equals(reference.getParentRef().toString())) {
                    return null;
                }
                return calcUpReference(reference.getParentRef(), rootRef, mappings, defaultQueryToAttach);
            }
        }
        return result;
    }

    private boolean inMappings(Set<String> mappings, Reference rootRef2, String upCandidate) {
        if (mappings == null || rootRef2 == null || upCandidate == null) {
            return false;
        }
        for (String path : mappings) {
            if (upCandidate.equals(rootRef2.toString() + path)) {
                return true;
            }
        }
        return false;
    }

    private String getDefaultQuery(Request request) {
        String result = "";
        if (request != null) {
            Reference originalRef = request.getOriginalRef();
            if (originalRef != null && originalRef.getQueryAsForm() != null) {
                Form queryAsForm = originalRef.getQueryAsForm();
                result = "?media=" + queryAsForm.getFirstValue("media");
            }
        }
        return result;
    }

    public String getBack() {
        return back;
    }

    public void setHome(String home) {
        this.home = home;
    }

    public String getHome() {
        return home;
    }

    public String getUp() {
        return up;
    }

}
